Package: CatsCradle
Title: This package provides methods for analysing spatial
        transcriptomics data and for discovering gene clusters
Version: 1.6.1
Authors@R: 
    c(person(given = "Anna",
        family = "Laddach",
        role = c("aut"),
        email = "anna.laddach@crick.ac.uk",
        comment = c(ORCID = "0000-0001-5552-6534")),
    person(given = "Michael",
        family = "Shapiro",
        role = c("aut","cre"),
        email = "michael.shapiro@crick.ac.uk",
        comment = c(ORCID = "0000-0002-2769-9320")))
Description: This package addresses two broad areas.  It allows for in-depth analysis of spatial transcriptomic data by identifying tissue neighbourhoods.  These are contiguous regions of tissue surrounding individual cells.  'CatsCradle' allows for the categorisation of neighbourhoods by the cell types contained in them and the genes expressed in them.  In particular, it produces Seurat objects whose individual elements are neighbourhoods rather than cells.  In addition, it enables the categorisation and annotation of genes by producing Seurat objects whose elements are genes.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Imports: Seurat (>= 5.0.1), ggplot2, networkD3, stringr, pracma,
        reshape2, rdist, igraph, geometry, Rfast, data.table, abind,
        pheatmap, EBImage, S4Vectors, SeuratObject,
        SingleCellExperiment, SpatialExperiment, Matrix, methods,
        SummarizedExperiment, msigdbr
Suggests: fossil, interp, knitr, BiocStyle, tictoc
Depends: R (>= 4.4.0)
LazyData: false
VignetteBuilder: knitr
BugReports: https://github.com/AnnaLaddach/CatsCradle/issues
URL: https://github.com/AnnaLaddach/CatsCradle
biocViews: BiologicalQuestion, StatisticalMethod, GeneExpression,
        SingleCell, Transcriptomics, Spatial
NeedsCompilation: no
Packaged: 2026-07-05 20:48:57 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/CatsCradle
git_branch: RELEASE_3_23
git_last_commit: c07e3c1
git_last_commit_date: 2026-07-02
Repository: Bioconductor 3.23
Date/Publication: 2026-07-05
Author: Anna Laddach [aut] (ORCID: <https://orcid.org/0000-0001-5552-6534>),
  Michael Shapiro [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2769-9320>)
Maintainer: Michael Shapiro <michael.shapiro@crick.ac.uk>
