# Generated by roxygen2: do not edit by hand

export(aggregateFeatureMatrix)
export(aggregateGeneExpression)
export(annotateGeneAsVector)
export(annotateGenesByGeneSet)
export(annotateLRInteractionCounts)
export(cellTypesPerCellTypeGraphFromCellMatrix)
export(cellTypesPerCellTypeGraphFromNbhdMatrix)
export(collapseExtendedNBHDs)
export(combinatorialSpheres)
export(computeCellTypesPerCellTypeMatrix)
export(computeEdgeGraph)
export(computeEdgeObject)
export(computeGraphEmbedding)
export(computeMoransI)
export(computeNBHDByCTMatrix)
export(computeNBHDVsCTObject)
export(computeNeighbourEnrichment)
export(computeNeighboursDelaunay)
export(computeNeighboursEuclidean)
export(countLRInteractionsPerCell)
export(cullEdges)
export(desymmetriseNN)
export(directedHausdorfDistance)
export(edgeCutoffsByClustering)
export(edgeCutoffsByPercentile)
export(edgeCutoffsByWatershed)
export(edgeCutoffsByZScore)
export(edgeLengthPlot)
export(edgeLengthsAndCellTypePairs)
export(exampleObjects)
export(geneSetsVsGeneClustersPValueMatrix)
export(getAverageExpressionDF)
export(getAverageExpressionMatrix)
export(getClusterOrder)
export(getExtendedNBHDs)
export(getFeatureZScores)
export(getGeneClusterAveragesPerCell)
export(getGeneNeighbors)
export(getLigandReceptorNetwork)
export(getLigandReceptorPairsInPanel)
export(getNearbyGenes)
export(getNearestNeighbourLists)
export(getObjectSubsetClusteringPValue)
export(getObjectSubsetClusteringStatistics)
export(getSubsetComponents)
export(make.getExample)
export(makeLRInteractionHeatmap)
export(makeSummedLRInteractionHeatmap)
export(meanGeneClusterOnCellUMAP)
export(meanZPerCluster)
export(meanZPerClusterOnUMAP)
export(medianComplementDistance)
export(medianComplementPValue)
export(nbhdsAsEdgesToNbhdsAsList)
export(neighbourhoodDiameter)
export(orderGeneSetPValues)
export(performLigandReceptorAnalysis)
export(plotLRDotplot)
export(predictAnnotation)
export(predictAnnotationAllGenes)
export(predictGeneAnnotationImpl)
export(randomiseGraph)
export(randomiseNodeIndices)
export(readGmt)
export(runGeometricClusteringTrials)
export(runMoransI)
export(sankeyFromMatrix)
export(stripGeneSet)
export(symmetriseNN)
export(symmetryCheckNN)
export(tagRowAndColNames)
export(transposeObject)
import(Seurat)
import(SeuratObject)
import(SingleCellExperiment)
import(SpatialExperiment)
import(ggplot2)
import(pheatmap)
importFrom(EBImage,watershed)
importFrom(Matrix,sparseMatrix)
importFrom(Rfast,colMins)
importFrom(Rfast,rowMins)
importFrom(Rfast,rowSort)
importFrom(S4Vectors,SelfHits)
importFrom(SummarizedExperiment,assays)
importFrom(abind,abind)
importFrom(data.table,data.table)
importFrom(data.table,merge.data.table)
importFrom(geometry,delaunayn)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlim)
importFrom(graphics,hist)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(methods,as)
importFrom(msigdbr,msigdbr)
importFrom(networkD3,sankeyNetwork)
importFrom(pracma,Norm)
importFrom(pracma,ceil)
importFrom(pracma,distmat)
importFrom(pracma,std)
importFrom(rdist,pdist)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,fisher.test)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,phyper)
importFrom(stats,sd)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_split_1)
importFrom(stringr,str_split_fixed)
importFrom(utils,data)
