% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exSeuratObj}
\alias{exSeuratObj}
\title{exSeuratObj}
\format{
A Seurat object

\describe{
A Seurat object of cells.  It includes a UMAP of the
cells and annotated clustering into cell types. It has
been severely reduced in size to accommodate Bioconductor
size restrictions.
}
}
\source{
This is subset from the data associated with
https://www.nature.com/articles/s41586-021-04006-z
}
\usage{
exSeuratObj
}
\description{
A Seurat object of 2000 genes by 540 cells.
}
\keyword{datasets}
